-- This module should be used to achieve an uniform and optimal
-- timing for the trace pins of a Nios II off-chip trace port.
-- Insert this module into the signal path between the output of
-- the SOPC block and the output pins.

-- This module only uses one extra logic cell in the core of the FPGA.
-- All other registers used in this module are implemented in the IO elements of
-- the FPGA and so don't use any core logic resources.
library IEEE;
use IEEE.std_logic_1164.all;

entity traceoutputbuffer is
	port (
		iClkX2     : in STD_LOGIC;

		iTraceClk  : in STD_LOGIC;
		iTraceData : in STD_LOGIC_VECTOR(17 downto 0);

		oTraceClk  : out STD_LOGIC;
		oTraceData : out STD_LOGIC_VECTOR(17 downto 0)
	);
	attribute useioff : boolean;
	attribute useioff of oTraceClk  : signal is true;
	attribute useioff of oTraceData : signal is true;
end traceoutputbuffer;

architecture behaviour of traceoutputbuffer is

-- ensure that rTraceClkTmp REALLY is instantiated by the synthesis.
attribute preserve : boolean;
signal rTraceClkTmp  : STD_LOGIC;
attribute preserve of rTraceClkTmp : signal is true;

signal rTraceClk     : STD_LOGIC;
signal rTraceData    : STD_LOGIC_VECTOR(17 downto 0);

begin
	oTraceClk<=rTraceClk;
	oTraceData<=rTraceData;
	process(iClkX2)
	begin
		-- Use iClkX2 to delay trace clock by one and a half cycle.
		-- This way the trace clock is center aligned to the data eyes.
		if iClkX2'event and iClkX2='0' then
			-- The usage of rTraceClkTmp is INTENTIONAL:
			-- The rTraceClkTmp register is fed directly by the output
			-- register of the SOPC block. Now this SOPC output register
			-- is clocked on the rising edge of iClkX2.
			-- So you have only 1/2 clock cycle to transfer the content of this
			-- output register to rTraceClkTmp.
			-- Because we use rTraceClkTmp, it can be fitted right next to
			-- the SOPC output register and so timing requirements are met easily.
			-- rTraceClk resides in the IO element, but for the transfer from rTraceClkTmp
			-- to rTraceClk one full clock cycle is available.
			rTraceClkTmp <= iTraceClk;
			rTraceClk    <= rTraceClkTmp;
		end if;
		-- Output registers in the IO element for the data signals.
		if iClkX2'event and iClkX2='1' then
			rTraceData <= iTraceData;
		end if;
	end process;
end behaviour;
